import pytest
import logging
import os
import csv
import configparser
from selenium.common import StaleElementReferenceException
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from 登录模块.base.Broswer_driver import Browser_driver

# 初始化日志记录器
logging.basicConfig(level=logging.ERROR, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def read_csv_data(file_path=None):
    """
    读取 CSV 文件数据
    """
    data = []
    if file_path is None:
        file_path = os.getenv('CSV_FILE_PATH', 'Login_Test.csv')

    try:
        with open(file_path, mode='r', encoding='utf-8') as csvfile:
            csv_reader = csv.DictReader(csvfile)
            for row in csv_reader:
                data.append((row['username'], row['password'], row['verification_code']))
    except FileNotFoundError:
        logger.error("CSV 文件未找到，请检查文件路径")
        pytest.exit("CSV 文件未找到，请检查文件路径")
    except csv.Error as e:
        logger.error(f"CSV 格式错误: {e}")
        pytest.exit(f"CSV 格式错误: {e}")
    except Exception as e:
        logger.error(f"出现错误: {e}")
        pytest.exit(f"出现错误: {e}")

    return data

@pytest.fixture(scope='session')
def driver():
    """
    初始化浏览器并打开指定URL
    """
    url = 'https://nat.ubainsyun.com:11046'
    logging.info("'----------' 正在初始化浏览器 '----------'")
    try:
        driver = Browser_driver().browser_init()
        logging.info("'----------' 浏览器初始化成功 '----------'")
    except Exception as e:
        logging.error(f"浏览器初始化失败: {e}")
        raise

    driver.implicitly_wait(10)
    driver.maximize_window()
    logging.info("'----------' 正在打开URL '----------'")

    try:
        driver.get(url)
        logging.info(f"成功打开URL: {url}")
    except Exception as e:
        logging.error(f"打开URL失败: {e}")
        raise

    handle_ssl_warning(driver)
    yield driver
    driver.quit()

def handle_ssl_warning(driver):
    """
    处理 SSL 证书警告
    """
    try:
        logging.info("'----------' 正在处理SSL证书警告 '----------'")
        WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="details-button"]'))
        ).click()

        WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="proceed-link"]'))
        ).click()

        WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.TAG_NAME, 'body'))
        )

        logging.info("'----------' SSL证书警告处理完成 '----------'")
    except Exception as e:
        logging.error(f"处理 SSL 证书警告遇到错误: {e}")
        raise

def _find_element(driver, by, value):
    """
    查找元素，并处理可能的异常
    """
    try:
        return WebDriverWait(driver, 20).until(
            EC.presence_of_element_located((by, value))
        )
    except StaleElementReferenceException:
        return WebDriverWait(driver, 20).until(
            EC.presence_of_element_located((by, value))
        )

def send_sms_verification_code(driver, phone_number):
    """
    模拟发送短信验证码
    """
    # 假设有一个发送短信验证码的API接口
    # 这里只是一个示例，实际应用中需要调用真实的API
    logging.info(f"模拟发送短信验证码至 {phone_number}")

def perform_login(driver, username, password, verification_code):
    """
    执行登录操作
    """
    driver.implicitly_wait(5)
    driver.refresh()

    try:
        username_input = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[1]/div/input')
        username_input.clear()
        username_input.send_keys(username)

        password_input = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[2]/div/input')
        password_input.clear()
        password_input.send_keys(password)

        # 假设验证码已经发送并接收
        send_sms_verification_code(driver, username)  # 发送验证码
        verification_input = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[3]/div[1]/div/input')
        verification_input.clear()
        verification_input.send_keys(verification_code)

        login_button = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[4]/input')
        login_button.click()

        if check_login_null(driver):
            logger.info("登录失败，必填项不能为空！断言成功！")
        elif check_login_success(driver):
            logger.info("登录成功，断言成功！")
            return_to_login_page(driver)
        elif check_login_failure(driver):
            logger.info("登录失败, 必填项错误！断言成功！")
        else:
            logger.error("登录状态未知，断言失败！")
            pytest.fail("登录状态未知，断言失败！")
    except StaleElementReferenceException as e:
        logger.error(f"元素已失效，重新查找... {e}")
        pytest.fail(f"元素已失效，重新查找... {e}")
    except Exception as e:
        logger.error(f"登录操作失败: {e}")
        pytest.fail(f"登录操作失败: {e}")

def return_to_login_page(driver):
    """
    返回登录页面
    """
    try:
        logout_button = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[1]/div/img[3]')
        logout_button.click()
        logging.info("已成功返回登录页面")
        driver.refresh()
        driver.implicitly_wait(5)
    except Exception as e:
        logger.error(f"返回登录页面失败: {e}")
        pytest.fail(f"返回登录页面失败: {e}")

def check_login_null(driver):
    """
    检查登录失败，必填项为空的情况
    """
    try:
        login_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, '/html/body/div[2]/p'))
        ).text
        assert "请输入" in login_element_text, "登录失败后未找到'请输入'提示语"
        return True
    except Exception as e:
        logger.error(f"登录失败检查失败: {e}")
        return False

def check_login_success(driver):
    """
    检查登录成功的情况
    """
    try:
        message_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, "//*[contains(text(), '欢迎')]"))
        ).text
        assert "欢迎 预定标准版测试" == message_element_text, "登录成功后找到'欢迎 预定标准版测试'提示语"
        return True
    except Exception as e:
        logger.error(f"登录成功检查失败: {e}")
        return False

def check_login_failure(driver):
    """
    检查登录失败的情况
    """
    try:
        login_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[1]/div[1]'))
        ).text
        assert "账号登录" == login_element_text, "登录失败后未找到'账号登录'提示语"
        return True
    except Exception as e:
        logger.error(f"登录失败检查失败: {e}")
        return False

@pytest.mark.parametrize("username,password,verification_code", read_csv_data())
def test_login_from_csv(driver, username, password, verification_code):
    """
    从 CSV 文件读取并测试登录情况
    """
    logger.info(f"'----------' 开始执行测试 '----------'")
    perform_login(driver, username, password, verification_code)
    logger.info(f"'----------' 执行完成测试 '----------'")
    driver.implicitly_wait(10)

if __name__ == '__main__':
    pytest.main()
